#include <QtGui>

#include "diagramscene.h"
#include "wire.h"
#include "devicedialog.h"


DiagramScene::DiagramScene(QMenu *itemMenu, QObject *parent)
    : QGraphicsScene(parent)
{
    myItemMenu = itemMenu;
    myMode = MoveItem;
    myItemType = DiagramDevice::Resistor;
	dynmPath = 0;
	statPath = 0;
    textItem = 0;
    myItemColor = Qt::white;
    myTextColor = Qt::black;
    myLineColor = Qt::black;
}


void DiagramScene::setLineColor(const QColor &color)
{
    myLineColor = color;
    if (isItemChange(Wire::Type)) {
        Wire *item =
            qgraphicsitem_cast<Wire *>(selectedItems().first());
        item->setColor(myLineColor);
        update();
    }
}


void DiagramScene::setTextColor(const QColor &color)
{
    myTextColor = color;
    if (isItemChange(DiagramTextItem::Type)) {
        DiagramTextItem *item =
            qgraphicsitem_cast<DiagramTextItem *>(selectedItems().first());
        item->setDefaultTextColor(myTextColor);
    }
}

//obsolete
void DiagramScene::setItemColor(const QColor &color)
{
    myItemColor = color;
 /*   if (isItemChange(DiagramDevice::Type)) {
        DiagramDevice *item =
            qgraphicsitem_cast<DiagramDevice *>(selectedItems().first());
        item->setBrush(myItemColor);
    }
*/
}


void DiagramScene::setFont(const QFont &font)
{
    myFont = font;

    if (isItemChange(DiagramTextItem::Type)) {
        QGraphicsTextItem *item =
            qgraphicsitem_cast<DiagramTextItem *>(selectedItems().first());
        //At this point the selection can change so the first selected item might not be a DiagramTextItem
        if (item)
            item->setFont(myFont);
    }
}


void DiagramScene::setMode(Mode mode)
{
    myMode = mode;
}

void DiagramScene::setItemType(DiagramDevice::DeviceType type)
{
    myItemType = type;
}


void DiagramScene::editorLostFocus(DiagramTextItem *item)
{
    QTextCursor cursor = item->textCursor();
    cursor.clearSelection();
    item->setTextCursor(cursor);

    if (item->toPlainText().isEmpty()) {
        removeItem(item);
        item->deleteLater();
    }
}


void DiagramScene::mousePressEvent(QGraphicsSceneMouseEvent *mouseEvent)
{
    if (mouseEvent->button() != Qt::LeftButton)
        return;

	DiagramDevice *device;//  = qgraphicsitem_cast<DiagramDevice *>(itemAt(mouseEvent->scenePos()));
	QList<QGraphicsItem *> devices = items(mouseEvent->scenePos());
	QPainterPath newPath;
	QPointF p;
    switch (myMode) {
        case InsertLine:
			/*state 1: If mouse clicks on a device and no wires are connected to cursor, a new wire will be generated 
				and connected to cursor. 
			  state 2: If mouse clicks on a device and currently there's a wire connected to the cursor, no actions will be performed.
			  state 3: If mouse clicks on the background, an intermediate point will be created on the wire, dividing it into two parts:
				a static path that is fixed on the canvas and connects device with the intermediate point,
				a dynamic path connecting the intermediate point with cursor and moving with cursor.
			*/
			if(dynmPath){
				if(!devices.isEmpty() && devices.last()->type() == DiagramDevice::Type)
					//press event will bee skipped here.
					break;//state 2

				//merge current dynamic path and previous static path into one new static path
				//state 3
				QPainterPath path(statPath->path());
				path.addPath(dynmPath->path());
				statPath->setPath(path);	
				dynmPath->path().moveTo(mouseEvent->scenePos());
				break;
			}

			//state 1
			device = qgraphicsitem_cast<DiagramDevice *>(itemAt(mouseEvent->scenePos()));
			
			if(itemAt(mouseEvent->scenePos()) == 0 || itemAt(mouseEvent->scenePos())->type() != DiagramDevice::Type)
				break;//wire can't be inserted in the blank area
            
			startPoint = mouseEvent->scenePos();

			p = device->mapFromScene(mouseEvent->scenePos());
			newPath.moveTo(device->mapToScene(device->getClosestPort(p)));

			dynmPath = new QGraphicsPathItem;//(QPainterPath(mouseEvent->scenePos()));
			dynmPath->setPen(QPen(Qt::blue, 2));
			dynmPath->setPath(newPath);
			addItem(dynmPath);

			statPath = new QGraphicsPathItem;//(QPainterPath(mouseEvent->scenePos()));
			statPath->setPen(QPen(Qt::blue, 2));
			statPath->setPath(newPath);
			addItem(statPath);

            break;

       case InsertText:
            textItem = new DiagramTextItem();
            textItem->setFont(myFont);
            textItem->setTextInteractionFlags(Qt::TextEditorInteraction);
            textItem->setZValue(1000.0);
            connect(textItem, SIGNAL(lostFocus(DiagramTextItem*)),
                    this, SLOT(editorLostFocus(DiagramTextItem*)));
            connect(textItem, SIGNAL(selectedChange(QGraphicsItem*)),
                    this, SIGNAL(itemSelected(QGraphicsItem*)));
            addItem(textItem);
            textItem->setDefaultTextColor(myTextColor);
            textItem->setPos(mouseEvent->scenePos());
            emit textInserted(textItem);
			break;

		case InsertDev:
			device = new DiagramDevice(myItemType, myItemMenu);
            addItem(device);
            device->setPos(mouseEvent->scenePos());
            emit itemInserted(device);
			break;
    default:
        ;
    }
	//Default routine of Qt is called at last.
    QGraphicsScene::mousePressEvent(mouseEvent);
}



void DiagramScene::mouseMoveEvent(QGraphicsSceneMouseEvent *mouseEvent)
{
    if (myMode == InsertLine && statPath != 0) {
		//the dynamic path connecting cursor with device moves will cursor
		//its location and shape will be updated repetitively
		QPainterPath newPath;
		newPath.moveTo(statPath->path().currentPosition());
		newPath.lineTo(QPointF(mouseEvent->scenePos().x(),statPath->path().currentPosition().y()));
		if(statPath->path().currentPosition().y() - mouseEvent->scenePos().y() > 7.0
			|| statPath->path().currentPosition().y() - mouseEvent->scenePos().y() < -7.0)
			newPath.lineTo(mouseEvent->scenePos());
		dynmPath->setPath(newPath);

    } else if (myMode == MoveItem) {
		//There's no self-defined action in this situation. Thus default routine of Qt is called.
        QGraphicsScene::mouseMoveEvent(mouseEvent);
    }// else if (myMode == InsertDev) {show device pixmap beside cursor}
}


void DiagramScene::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *mouseEvent)
{
	if(myMode == MoveItem){//create property window of device item
		QGraphicsItem *Item = itemAt(mouseEvent->scenePos());
		if(Item == 0 || Item->type() == Wire::Type) return;
		DeviceDialog *devicedialog = new DeviceDialog(qgraphicsitem_cast<DiagramDevice *>(Item));
		devicedialog->exec();
		delete devicedialog;
		return;
	}

	//Connect two devices
    if (dynmPath != 0 && myMode == InsertLine) {
        QList<QGraphicsItem *> startItems = items(startPoint);
        while (startItems.count() && startItems.first() == statPath)
            startItems.removeFirst();
        QList<QGraphicsItem *> endItems = items(mouseEvent->scenePos());
        while (endItems.count() && endItems.first() == dynmPath)
            endItems.removeFirst();


		if (startItems.count() > 0 && endItems.count() > 0 &&
            startItems.first()->type() == DiagramDevice::Type &&
            endItems.first()->type() == DiagramDevice::Type &&
            startItems.first() != endItems.first()) {
            DiagramDevice *startDevice =
                qgraphicsitem_cast<DiagramDevice *>(startItems.first());
            DiagramDevice *endDevice =
                qgraphicsitem_cast<DiagramDevice *>(endItems.first());
            Wire *wire = new Wire(startDevice, endDevice);
			wire->setStartPortIndex(startDevice->mapFromScene(startPoint));
			wire->setEndPortIndex(endDevice->mapFromScene(mouseEvent->scenePos()));
            wire->setColor(myLineColor);
			startDevice->addWire(wire,wire->startPortIndex());
			endDevice->addWire(wire,wire->endPortIndex());
            wire->setZValue(-1000.0);

			DiagramDevice *device = qgraphicsitem_cast<DiagramDevice *>(endItems.first());
			QPointF p = device->mapFromScene(mouseEvent->scenePos());
			QPainterPath newPath(statPath->path());
			newPath.lineTo(device->mapToScene(device->getClosestPort(p)).x(),newPath.currentPosition().y());
			newPath.lineTo(device->mapToScene(device->getClosestPort(p)));
			statPath->setPath(newPath);
			
            addItem(wire);
			wire->setPath(statPath->path());
        }

		removeItem(dynmPath);
		delete dynmPath;

		removeItem(statPath);
		delete statPath;
    }

	dynmPath = statPath = 0;

	//Default function is called
    QGraphicsScene::mouseReleaseEvent(mouseEvent);
}



bool DiagramScene::isItemChange(int type)
{
    foreach (QGraphicsItem *item, selectedItems()) {
        if (item->type() == type)
            return true;
    }
    return false;
}



